### PersefoneDesktop - a GUI to the Persefone model of agriculture and ecosystems
###
### This file links the QML UI designs to the underlying model.
###

## For references, see:
## - https://github.com/barche/QML.jl
## - https://github.com/barche/QmlJuliaExamples
## - https://doc.qt.io/qt-6/qtquick-index.html

global model::Union{AgentBasedModel,Missing} = missing
#loadmodelobject(joinpath(dirname(@__FILE__), "../sampleinit.dat")) #initialise()
const running = Observable(false)
const date = Observable(today())
const progress = Observable(0.0)
const delay = Observable(0.5)
const runbuttontext = Observable(">>")
const runbuttontip = Observable("Run")

function newsimulation()
    global model
    running[] = false
    progress[] = 0.0
    model = initialise() #TODO configure
    date[] = model.date
    println("Model initialised.")
end

#TODO loadsimulation()
#TODO saveoutput()

function nextstep()
    global model
    stepsimulation!(model)
    date[] = model.date
    progress[] = (model.date-@param(core.startdate)) / (@param(core.enddate)-model.date)
    println("Updated model: $(model.date-Day(1))")
end

function runsimulation()
    global model
    while running[] #FIXME how can I interrupt this?
        stepsimulation!(model)
        date[] = model.date
        progress[] = (model.date-@param(core.startdate)) / (@param(core.enddate)-model.date)
        sleep(delay)
        println("Updated model: $(model.date-Day(1))")
    end
end

function togglerunning()
    if running[]
        running[] = false
        runbuttontext[] = ">>"
        runbuttontip[] = "Run"
    else
        running[] = true
        runbuttontext[] = "||"
        runbuttontip[] = "Pause"
        runsimulation()
    end
end
    
#TODO previousstep() - needs adjustments to nextstep() and runsimulation()

function render_map(screen)
    global model
    figure = visualisemap(model)
    display(screen, figure.scene)
end

@qmlfunction newsimulation
@qmlfunction nextstep
@qmlfunction togglerunning
@qmlfunction render_map

on(delay) do d
    println("Delay is now $(round(d, digits=1)) seconds.")
end

on(running) do r
    r ? println("Simulation started.") : println("Simulation stopped.")
end

"""
    launch()

The main function that creates the application.
"""
function launch()
    # absolute path in case working dir is overridden
    qml_file = joinpath(dirname(@__FILE__), "main.qml")
    loadqml(qml_file,
            vars = JuliaPropertyMap("running" => running,
                                    "date" => date,
                                    "delay" => delay,
                                    "progress" => progress,
                                    "runbuttontext" => runbuttontext,
                                    "runbuttontip" => runbuttontip))
    println("Launched Persefone Desktop.")
    exec()
end
